/*  
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MPG 
#define MPG 

#include <list>

#define MAX 1 // player-1 is MAX
#define MIN 0 // player-0 is MIN

/* MPG weighted arc data type definition */
struct t_w_arc{
	unsigned int head_idx;
	long weight;
};

struct t_rational{
	long long num;
	long long denom;
};

/* MPG data type definition */
class MeanPayoffGame{
	private:
	unsigned int n_0; // number of V_0 black nodes
	unsigned int n_1; // number of V_1 white nodes
	unsigned int e; // number of arcs
	// we assume that black nodes are numbered 0 to n_0-1
	// and that white nodes are numbered n_0 to n_0 + n_1 - 1
	std::list<t_w_arc*>* arcs; // array of adjacency lists  
	
	void load(const char* f);
	void init_arcs();
	void delete_arcs();
	long long *compute_nu_k(unsigned int s, unsigned long long k);
	t_rational search_for_rational(long long n, unsigned long k);
	public:
	MeanPayoffGame(const char* f);
	MeanPayoffGame(unsigned int n_0, unsigned int n_1);
	~MeanPayoffGame();

 	bool* ZP_solve_decision(); // Zwick & Paterson decision algorithm
	t_rational* ZP_solve_opt_values();	

	std::list<t_w_arc*>* get_arcs(unsigned int t);
	void set_arcs(unsigned int u, std::list<t_w_arc*>* l);
	void push_arc(unsigned int t, t_w_arc* e);
	unsigned int get_n_0();
	unsigned int get_n_1();
	unsigned int get_e();
	void remove_arc(unsigned int u);
	unsigned long get_W();	
	bool is_well_defined();
	void print();	
};

#endif
